/**
   @file ndxlbl.h
   @brief Header for structure NdxLbl, which keeps track of the relationship
   between labels and the indices that refer to them. 
   @internal
   Copyright (C) 2002 Alan R. Rogers

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Alan R. Rogers, Department of Anthropology, University of Utah,
   Salt Lake City, UT 84112. rogers@anthro.utah.edu
**/
#ifndef CCONTAIN_NDXLBL_H
#define CCONTAIN_NDXLBL_H

#include "poco/pocofwd.h"
#include "poco/bintree.h"

/**
 * NdxLbl handles the mapping between the external names and internal
 * indices.
 **/

/** default constructor */
NdxLbl * 
NdxLbl_construct(void);

void
NdxLbl_destroy(NdxLbl * nl);

/** Insert lbl into NdxLbl object, incrementing count if lbl is already
 * there.  Return 0 on success, nonzero on failure. */
int
NdxLbl_insert(NdxLbl * nl, const char * lbl);

/**
 * Find index of label lbl, making a new entry if necessary.
 * @param lbl A pointer to a string containing a label.
 * @return index of lbl
 **/
int
NdxLbl_ndx(NdxLbl *nl, const char * lbl);

/**
 * @param ndx the (integer) index of a group
 * @return a pointer to a newly allocated string containing the group
 * label associated with ndx.  If ndx isn't found, 0 is returned.
 **/
char * 
NdxLbl_lbl(NdxLbl *nl, int ndx);

/** Return the number of label/index pairs stored stored */
int 
NdxLbl_size(NdxLbl *nl);

/** Return the number of observations with given label */
int
NdxLbl_countLbl(NdxLbl * nl, const char * lbl);

/** Return the number of observations with given index */
int
NdxLbl_countNdx(NdxLbl * nl, int ndx);

#endif /* NDXLBL_H */
