
/*
 * These functions handle errors and then terminate execution.  They
 * are copied from pp 109-114 of: Brian Kernighan and Rob
 * Pike. 1997. The Practice of Programming.  Addison-Wesley.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include "poco/err.h"

/* eprintf: print error message and exit */
void
poco_eprintf(const char *fmt, ...) {
    va_list         args;

    fflush(stdout);

    va_start(args, fmt);
    vfprintf(stdout, fmt, args);
    va_end(args);

    if(fmt[0] != '\0' && fmt[strlen(fmt) - 1] == ':')
        fprintf(stdout, " %s", strerror(errno));
    fprintf(stdout, "\n");
    exit(2);                    /* conventional value for failed execution */
}

/* estrdup: duplicate a string, report if error */
char *
poco_estrdup(const char *file, int line, const char *s) {
    char           *t;

    if(s == NULL)
	return NULL;
    
    t = (char *) malloc(strlen(s) + 1);
    if(t == NULL)
       poco_eprintf("estrdup(\"%.20s\") failed @%s:%d", s, file, line);
    strcpy(t, s);
    return t;
}

/** emalloc: malloc and report if error */
void *
poco_emalloc(const char *file, int line, size_t n) {
    void           *p;

    if(n == 0)
	return NULL;

    p = malloc(n);
    if(p == NULL)
        poco_eprintf("emalloc( %u ) failed @%s:%d", n, file, line);
    return p;
}
