#include "poco/vector.h"
#include <stdio.h>
#include <stdlib.h>

/*
 * Resize vector to new size n, copying contents to new location if
 * necessary.  Return 0 on success; nonzero on failure
 */
int
Vector_resize( Vector * v, int n ) {
    int i;

    if(n <= v->size) {
	/* no-op if n == size */
	for(i=n; i < v->size; ++i) {
	    v->eif->destroy(v->base[i]);
	    v->base[i] = NULL;
	}
	v->size = n;
	return 0;
    }

    /* else n > v->size */
    if(n > v->capacity) {
	if( Vector_reserve( v, n ) != 0 )
	    return 1;
    }
    for(i=v->size; i < n; ++i)
	v->base[i] = v->eif->construct(v->eif->param);
    v->size = n;
    return 0;
}

/*
 * Increase capacity, the number of void pointers allocated.
 * No-op if n < current capacity.
 * Return zero on success, non-zero on failure.
 */
int
Vector_reserve( Vector * v, int n ) {
    if(n <= v->capacity)
	return 0;

    v->base = (void **) realloc(v->base, n * sizeof(void *));
    if(v->base == NULL) {
	v->size = v->capacity = 0;
	return 1;
    }
    v->capacity = n;
    return 0;
}

