#include "poco/element.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/* Test BinTree with data type string */
void * strCpy2(const void * data);
int strCompare2(const void *lpp, const  void *rpp);
void strDestroy2(void * s);
void tstElement(int verbose);

void * strCpy2(const void * data)
{
    char * to;
    const char * from = (const char *) data;
    if(from == NULL)
	return NULL;

    to = strdup(from);

    return (void *) to;
}

int strCompare2(const void *lpp, const void *rpp)
{
    const char * l = *((char * const *) lpp);
    const char * r = *((char * const *) rpp);
    return strcmp(l,r);
}

void strDestroy2(void * s)
{
    free(s);
}

void
tstElement(int verbose) {
    ElementInterface * eif = ElementInterface_construct(NULL,
							strCompare2,
							NULL,
							strCpy2,
							strDestroy2,
							NULL);
    char * str1 = strdup("test string");
    char * str2 = eif->copy(str1);

    if(strcmp(str1, str2) == 0) {
	if(verbose)
	    printf("String copy is OK\n");
    } else {
	if(verbose) {
	    printf("ElementInterface is NOT ok\n");
	    printf("  eif->copy didn't produce an identical string\n");
	}
	exit(1);
    }

    if(eif->comparepp(&str1, &str2) == 0) {
	if(verbose)
	    printf("String comparison is OK\n");
    } else {
	if(verbose) {
	    printf("ElementInterface is NOT ok\n");
	    printf("  eif->compare returned non-zero on identical strings\n");
	}
	exit(1);
    }

    eif->destroy(str2);

    /* destroy object */
    ElementInterface_destroy(eif);

    if(verbose)
	printf("tstElement OK\n");

    return;
}
