#include "poco/bintree.h"
#include "poco/err.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/* Test BinTree with data type string */
void * strCpy(const void * data);
int strCompare(const void *lpp, const  void *rpp);
void strDel(void * s);

void * strCpy(const void * data)
{
    char * to;
    const char * from = (const char *) data;
    if(from == NULL)
	return NULL;

    to = strdup(from);

    return (void *) to;
}

int strCompare(const void *lpp, const void *rpp)
{
    const char * l = *((char * const *) lpp);
    const char * r = *((char * const *) rpp);
    return strcmp(l,r);
}

void strDel(void * s)
{
    free(s);
}

/**
   Test BinTree and BinTreeNode.  Use -v arg for printed output.
**/
int main(int argc, char ** argv)
{
    int verbose = 0;
    ElementInterface * eif;
    BinTree * bt;
    BinTreeNode *n;

    if(argc > 1 && strcmp(argv[1], "-v") == 0)
	verbose = 1;

    eif = ElementInterface_construct(NULL, strCompare, NULL, strCpy, strDel,
				     NULL);
    assert(eif);

    bt = BinTree_construct(eif);
    
    /* Add things to tree */
    assert( BinTree_insert( bt, "foo"));
    assert( BinTree_insert( bt, "bar" ));
    assert( BinTree_insert( bt, "foo" ));
    n = BinTree_insert( bt, "foo" );
    assert(n);
    assert(strcmp("foo", BinTreeNode_value(n)) == 0);
    assert(BinTreeNode_count(n) == 3);
    assert( BinTree_insert( bt, "foobar" ));

    /* Check that counts are correct */
    n = BinTree_find( bt, "foo");
    if(n == NULL) {
	poco_eprintf("Couldn't find %s\n", "foo");
    }
    if( BinTreeNode_count(n) != 3 ) {
	poco_eprintf("Count(%s) != %d\n", "foo", 3);
    }

    n = BinTree_find( bt, "bar" );
    if(n == NULL) {
	poco_eprintf("Couldn't find %s\n", "bar");
    }
    if( BinTreeNode_count(n) != 1 ) {
	poco_eprintf("Count(%s) != %d\n", "bar", 1);
    }

    n = BinTree_find( bt, "foobar" );
    if(n == NULL) {
	poco_eprintf("Couldn't find %s\n", "foobar");
    }
    if( BinTreeNode_count(n) != 1 ) {
	poco_eprintf("Count(%s) != %d\n", "foobar", 1);
    }

    /* Walk through tree */
    n = BinTree_min(bt);
    while(n != NULL) {
	if(verbose)
	    printf("  count(%s)=%d\n", (char*) BinTreeNode_value(n),
		   BinTreeNode_count(n));
	n = BinTreeNode_next(n);
    }

    /* delete tree */
    BinTree_destroy(bt);
    ElementInterface_destroy(eif);

    printf("BinTree OK\n");
    return 0;
}
