#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#define ALLOC(l,bytes) (l)=malloc(bytes); \
  if((l)==NULL){\
    fprintf(stderr,"malloc failed @%s:%d\n", __FILE__,__LINE__);\
    exit(1);\
  }

#include "poco/dblvec.h"

/** Test nvec.h.  Use -v arg for printed output. **/
int main(int argc, char ** argv)
{
    int verbose = 0;
    int i;
    DblVec * v1, * v2;

    if(argc > 1 && strcmp(argv[1], "-v") == 0)
	verbose = 1;

    v1 = DblVec_construct(0);
    DblVec_destroy(v1);

    v1 = DblVec_construct(10);
    assert(DblVec_size(v1) == 0);
    assert(DblVec_capacity(v1) == 10);
    assert(DblVec_sum(v1) == 0.0);

    DblVec_pushBack( v1, 2.0 );
    assert(DblVec_size(v1) == 1);
    assert(DblVec_capacity(v1) == 10);
    assert(DblVec_at( v1, 0 ) == 2.0);
    assert(DblVec_sum(v1) == 2.0);

    if(verbose) {
	printf("v1:");
	DblVec_print(v1, stdout);
    }

    DblVec_resize( v1, 4 );
    assert(DblVec_size(v1) == 4);
    assert(DblVec_capacity(v1) == 10);
    DblVec_setValue( v1, 3, 3.0 );
    assert(DblVec_sum(v1) == 5.0);

    if(verbose) {
	printf("After assignment, v1:");
	DblVec_print(v1, stdout);
    }

    DblVec_reserve( v1, 30 );
    assert(DblVec_size(v1) == 4);
    assert(DblVec_capacity(v1) == 30);

    DblVec_clear(v1);
    assert(DblVec_size(v1) == 0);
    assert(DblVec_capacity(v1) == 30);

    DblVec_destroy(v1);     /* destroy empty vector */

    v1 = DblVec_construct(0);
    DblVec_reserve( v1, 3);
    DblVec_pushBack( v1, 1.0 );
    DblVec_pushBack( v1, 2.0 );
    DblVec_pushBack( v1, 3.0 );

    assert(DblVec_size(v1) == 3);
    assert(DblVec_popBack(v1) == 3.0);
    assert(DblVec_size(v1) == 2);
    DblVec_pushBack( v1, 3.0 );

    v2 = DblVec_construct(5);
    DblVec_pushBack( v2, 8.0 );
    DblVec_pushBack( v2, 9.0 );
    DblVec_pushBack( v2, 10.0 );

    assert(DblVec_comparepp(&v1, &v2) < 0.0);
    assert(DblVec_comparepp(&v2, &v1) > 0.0);
    assert(DblVec_comparepp(&v1, &v1) == 0.0);
    assert(DblVec_compare(v1, v2) < 0.0);

    DblVec_append( v1, v2 );
    assert(DblVec_size(v1) == 6);
    assert(DblVec_size(v2) == 3);
    assert(DblVec_at( v2, 1) == DblVec_at( v1, 4 ));
    DblVec_assign(v2, v1);
    assert(DblVec_compare(v2, v1) == 0);
    DblVec_destroy(v2);

    v2 = DblVec_copy(v1);
    assert(DblVec_size(v1) == DblVec_size(v2));
    for(i=0; i< DblVec_size(v1); ++i) {
	assert(DblVec_at( v1, i ) == DblVec_at( v2, i ));
	DblVec_increment( v2, i );
	assert(1+DblVec_at( v1, i ) == DblVec_at( v2, i ));
    }

    DblVec_clear(v1);
    DblVec_pushBack(v1, 3.0);
    DblVec_pushBack(v1, 2.1);
    DblVec_pushBack(v1, 5.2);
    DblVec_pushBack(v1, 4.2);
    DblVec_pushBack(v1, 3.0);

    assert(!DblVec_isSorted(v1));
    DblVec_sort(v1);
    if(verbose) {
	printf("After sort:");
	DblVec_print(v1, stdout);
    }

    assert(DblVec_isSorted(v1));
    assert(DblVec_lookup(v1, -2.0) == 0);
    assert(DblVec_lookup(v1, 0.0) == 0);
    assert(DblVec_lookup(v1, 3.0) == 1);
    assert(DblVec_lookup(v1, 4.2) == 3);
    assert(DblVec_lookup(v1, 5.2) == 4);
    assert(DblVec_lookup(v1, 6.0) == -1);

    DblVec_destroy(v1);     /* destroy full vector */
    DblVec_destroy(v2);

    printf("DblVec OK\n");
    return 0;
}
