#include "poco/element.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/* Test BinTree with data type string */
void * strCpy(const void * data);
int strComparepp(const void *lpp, const  void *rpp);
void strDestroy(void * s);
void tstElement(int verbose);

void * strCpy(const void * data)
{
    char * to;
    const char * from = (const char *) data;
    if(from == NULL)
	return NULL;

    to = strdup(from);

    return (void *) to;
}

int strComparepp(const void *lpp, const void *rpp)
{
    const char * l = *((const char * const *) lpp);
    const char * r = *((const char * const *) rpp);
    return strcmp(l,r);
}

void strDestroy(void * s)
{
    free(s);
}

/**
   Test ElementInterface.  Use -v arg for printed output.
**/
int main(int argc, char ** argv)
{
    int verbose = 0;
    ElementInterface * eif = ElementInterface_construct(NULL,
							strComparepp,
							NULL,
							strCpy,
							strDestroy,
							NULL);
    ElementInterface * eif2 = ElementInterface_copy(eif);
    char * str1 = strdup("test string");
    char * str2 = eif->copy(str1);

    if(argc > 1 && strcmp(argv[1], "-v") == 0)
	verbose = 1;

    if(strcmp(str1, str2) == 0) {
	if(verbose)
	    printf("String copy is OK\n");
    } else {
	printf("ElementInterface is NOT ok\n");
	printf("  eif->copy didn't produce an identical string\n");
	exit(1);
    }

    if(eif->comparepp((const void **) &str1, &str2) != 0) {
	printf("ElementInterface is NOT ok\n");
	printf("  eif->compare!=0 on identical strings\n");
	exit(1);
    }

    if(ElementInterface_comparepp(eif, &str1, &str2) != 0) {
	printf("ElementInterface is NOT ok\n");
	printf("  ElementInterface_comparepp!=0 on identical strings\n");
	exit(1);
    }

    if(ElementInterface_compare(eif, str1, str2) != 0) {
	printf("ElementInterface is NOT ok\n");
	printf("  ElementInterface_compare!=0 on identical strings\n");
	exit(1);
    }

    eif->destroy(str2);

    str2 = eif2->copy(str1);
    assert(strcmp(str1, str2) == 0);

    /* destroy object */
    ElementInterface_destroy(eif);
    ElementInterface_destroy(eif2);

    tstElement(verbose);

    printf("ElementInterface OK\n");
    return 0;
}
