#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#define ALLOC(l,bytes) (l)=malloc(bytes); \
  if((l)==NULL){\
    fprintf(stderr,"malloc failed @%s:%d\n", __FILE__,__LINE__);\
    exit(1);\
  }

#include "poco/intvec.h"

/** Test nvec.h.  Use -v arg for printed output. **/
int main(int argc, char ** argv)
{
    int verbose = 0;
    int i;
    IntVec * v1, * v2;

    if(argc > 1 && strcmp(argv[1], "-v") == 0)
	verbose = 1;

    v1 = IntVec_construct(0);
    IntVec_destroy(v1);

    v1 = IntVec_construct(10);

    assert(IntVec_size(v1) == 0);
    assert(IntVec_capacity(v1) == 10);

    IntVec_pushBack(v1, 2);
    assert(IntVec_size(v1) == 1);
    assert(IntVec_capacity(v1) == 10);
    assert(IntVec_sum(v1) == 2);
    assert(IntVec_at(v1, 0) == 2);
    IntVec_pushBack(v1, 3);
    assert(IntVec_sum(v1) == 5);

    if(verbose) {
	printf("IntVec_size(v1)=%d\n", IntVec_size(v1));
	printf("v1:");
	IntVec_print(v1, stdout);
    }

    IntVec_resize(v1, 4);
    assert(IntVec_sum(v1) == 5);
    assert(IntVec_size(v1) == 4);
    assert(IntVec_capacity(v1) == 10);
    IntVec_setValue(v1, 3, 3);
    assert(IntVec_sum(v1) == 8);

    if(verbose) {
	printf("After assignment, v1:");
	IntVec_print(v1, stdout);
    }

    IntVec_reserve(v1, 30);
    assert(IntVec_size(v1) == 4);
    assert(IntVec_capacity(v1) == 30);

    IntVec_clear(v1);
    assert(IntVec_size(v1) == 0);
    assert(IntVec_capacity(v1) == 30);

    IntVec_destroy(v1);     /* destroy empty vector */

    v1 = IntVec_construct(0);
    IntVec_reserve(v1, 3);
    IntVec_pushBack(v1, 1);
    IntVec_pushBack(v1, 2);
    IntVec_pushBack(v1, 3);
    assert(IntVec_size(v1) == 3);

    assert(IntVec_popBack(v1) == 3);
    assert(IntVec_size(v1) == 2);
    IntVec_pushBack(v1, 3);

    v2 = IntVec_construct(5);
    IntVec_pushBack(v2, 8);
    IntVec_pushBack(v2, 9);
    IntVec_pushBack(v2, 10);
    assert(IntVec_size(v2) == 3);

    assert(IntVec_comparepp(&v1, &v2) > 0);
    assert(IntVec_comparepp(&v2, &v1) < 0);
    assert(IntVec_comparepp(&v1, &v1) == 0);
    assert(IntVec_compare(v1, v2) > 0);

    IntVec_append(v1, v2);
    assert(IntVec_compare(v1, v2) != 0);
    assert(IntVec_compare(v1, v2) == -IntVec_compare(v2, v1));
    assert(IntVec_size(v1) == 6);
    assert(IntVec_size(v2) == 3);
    assert(IntVec_at(v2, 1) == IntVec_at(v1, 4));

    IntVec_assign(v2, v1);
    assert(IntVec_compare(v2, v1) == 0);
    assert(IntVec_compare(v1, v2) == 0);

    IntVec_destroy(v2);

    v2 = IntVec_copy(v1);
    assert(IntVec_size(v1) == IntVec_size(v2));
    assert(IntVec_capacity(v2) == IntVec_size(v2));
    for(i=0; i< IntVec_size(v1); ++i) {
	assert(IntVec_at(v1, i) == IntVec_at(v2, i));
	IntVec_increment(v2, i, 1);
	assert(1+IntVec_at(v1, i) == IntVec_at(v2, i));
    }

    IntVec_clear(v1);
    IntVec_pushBack(v1, 3);
    IntVec_pushBack(v1, 2);
    IntVec_pushBack(v1, 5);
    IntVec_pushBack(v1, 4);
    IntVec_pushBack(v1, 3);

    assert(!IntVec_isSorted(v1));
    IntVec_sort(v1);

    assert(IntVec_isSorted(v1));
    assert(IntVec_lookup(v1, -2) == 0);
    assert(IntVec_lookup(v1, 0) == 0);
    assert(IntVec_lookup(v1, 3) == 1);
    assert(IntVec_lookup(v1, 4) == 3);
    assert(IntVec_lookup(v1, 5) == 4);
    assert(IntVec_lookup(v1, 6) == -1);

    IntVec_destroy(v1);     /* destroy full vector */
    IntVec_destroy(v2);

    printf("IntVec OK\n");
    return 0;
}
