#include "poco/ndxlbl.h"
#include <stdio.h>
#include <string.h>
#include <assert.h>

int main(int argc, char ** argv)
{
    int verbose = 0;
    NdxLbl * nl = NdxLbl_construct();
    char * sAfrica = strdup("Africa");
    char * sAsia = strdup("Asia");
    char * sEurope = strdup("Europe");
    const int iAfrica=0, iAsia=1, iEurope=2;
    int ndx;
    
    if(argc == 2 && strcmp(argv[1], "-v") == 0)
	verbose = 1;
    
    /* Insert strings */
    (void) NdxLbl_insert(nl, sAfrica);  /* Africa once */

    assert(NdxLbl_ndx(nl, sAsia) == -1);   /* Asia isn't there yet */

    (void) NdxLbl_insert(nl, sAsia);      /* Asia 3 times */
    (void) NdxLbl_insert(nl, sAsia);
    (void) NdxLbl_insert(nl, sAsia);

    (void) NdxLbl_insert(nl, sEurope);  /* Europe twice */
    ndx = NdxLbl_insert(nl, sEurope);
    assert(ndx == 2);
    
    if(verbose) {
	printf("  ndx[%s] = %u\n", sAfrica, NdxLbl_ndx(nl, sAfrica));
	printf("  ndx[%s] = %u\n", sAsia, NdxLbl_ndx(nl, sAsia));
	printf("  ndx[%s] = %u\n", sEurope, NdxLbl_ndx(nl, sEurope));
	printf("  lbl[%u] = %s\n", iAfrica, NdxLbl_lbl(nl, iAfrica));
	printf("  lbl[%u] = %s\n", iAsia, NdxLbl_lbl(nl, iAsia));
	printf("  lbl[%u] = %s\n", iEurope, NdxLbl_lbl(nl, iEurope));
	printf("  count[%s] = %u\n", sAfrica, NdxLbl_countLbl(nl,sAfrica));
	printf("  count[%s] = %u\n", sAsia, NdxLbl_countLbl(nl,sAsia));
	printf("  count[%s] = %u\n", sEurope, NdxLbl_countLbl(nl,sEurope));
	printf("  count[%u] = %u\n", iAfrica, NdxLbl_countNdx(nl,iAfrica));
	printf("  count[%u] = %u\n", iAsia, NdxLbl_countNdx(nl,iAsia));
	printf("  count[%u] = %u\n", iEurope, NdxLbl_countNdx(nl,iEurope));
    }

    /* Check indices */
    assert(iAfrica == NdxLbl_ndx(nl,sAfrica));
    assert(iAsia == NdxLbl_ndx(nl, sAsia));
    assert(iEurope == NdxLbl_ndx(nl, sEurope));

    /* Check labels */
    assert(strcmp(sAfrica, NdxLbl_lbl(nl, iAfrica)) == 0);
    assert(strcmp(sAsia, NdxLbl_lbl(nl, iAsia)) == 0);
    assert(strcmp(sEurope, NdxLbl_lbl(nl, iEurope)) == 0);
    
    /* Check counts */
    assert(NdxLbl_countLbl(nl, sAfrica) == 1u);
    assert(NdxLbl_countLbl(nl, sAsia) == 3u);
    assert(NdxLbl_countLbl(nl, sEurope) == 2u);
    assert(NdxLbl_countNdx(nl, iAfrica) == 1u);
    assert(NdxLbl_countNdx(nl, iAsia) == 3u);
    assert(NdxLbl_countNdx(nl, iEurope) == 2u);

    printf("NdxLbl OK\n");
    return 0;
}

